@@ -1,4 +1,4 @@
-/* Copyright 2007, 2008 Matthew Barnes
+/* Copyright 2007-2010 Matthew Barnes
  *
  * This file is part of GNOME Video Arcade.
  *
@@ -24,6 +24,7 @@
 #include "gva-cell-renderer-pixbuf.h"
 #include "gva-error.h"
 #include "gva-favorites.h"
+#include "gva-nplayers.h"
 #include "gva-tree-view.h"
 #include "gva-ui.h"
 
@@ -169,7 +170,7 @@ columns_driver_status_set_properties (GtkTreeViewColumn *column,
                 -1);
 
         /* XXX For the "good" icon I want some kind of positive symbol; a
-         *     checkmark or thumbs up or something.  The GTK_STOCK_APPLY 
+         *     checkmark or thumbs up or something.  The GTK_STOCK_APPLY
          *     image seems to be the closest match at this time (a green
          *     checkmark), but may not be suitable for all icon themes. */
 
@@ -243,21 +244,112 @@ columns_time_set_properties (GtkTreeViewColumn *column,
 {
         GvaGameStoreColumn column_id;
         GValue value;
-        gchar text[256];
+        gchar text[256] = { '\0' };
+        time_t *time_ptr;
 
         memset (&value, 0, sizeof (GValue));
         column_id = gtk_tree_view_column_get_sort_column_id (column);
         gtk_tree_model_get_value (model, iter, column_id, &value);
 
-        strftime (
-                text, sizeof (text), nl_langinfo (D_T_FMT),
-                localtime (g_value_get_boxed (&value)));
+        time_ptr = g_value_get_boxed (&value);
+
+        /* Render epoch values as empty cells. */
+        if (*time_ptr > (time_t) 0)
+        {
+                GDate date, today;
+                gboolean date_is_today;
+
+                g_date_clear (&date, 1);
+                g_date_clear (&today, 1);
+
+                g_date_set_time_t (&date, *time_ptr);
+                g_date_set_time_t (&today, time (NULL));
+
+                date_is_today = (g_date_compare (&date, &today) == 0);
+
+                strftime (
+                        text, sizeof (text),
+                        nl_langinfo (date_is_today ? T_FMT : D_FMT),
+                        localtime (time_ptr));
+        }
 
         g_object_set (renderer, "text", text, NULL);
 
         g_value_unset (&value);
 }
 
+static gboolean
+columns_show_popup_menu (GdkEventButton *event,
+                         GtkTreeViewColumn *column)
+{
+        GtkMenu *menu;
+        GtkWidget *view;
+        const gchar *column_title;
+        gchar *label, *tooltip;
+
+        column_title = gtk_tree_view_column_get_title (column);
+        view = gtk_tree_view_column_get_tree_view (column);
+
+        /* Let the "Add Column" action know where to insert the column. */
+        g_object_set_data (G_OBJECT (view), "popup-menu-column", column);
+
+        /* Update the "Remove Column" item in the popup menu. */
+        label = g_strdup_printf (
+                _("Remove %s Column"), column_title);
+        tooltip = g_strdup_printf (
+                _("Remove the \"%s\" column from the game list"),
+                column_title);
+        g_object_set (
+                GVA_ACTION_REMOVE_COLUMN, "label",
+                label, "tooltip", tooltip, NULL);
+        g_free (tooltip);
+        g_free (label);
+
+        menu = GTK_MENU (gva_ui_get_managed_widget ("/column-popup"));
+
+        if (event != NULL)
+                gtk_menu_popup (
+                        menu, NULL, NULL, NULL, NULL,
+                        event->button, event->time);
+        else
+                gtk_menu_popup (
+                        menu, NULL, NULL, NULL, NULL,
+                        0, gtk_get_current_event_time ());
+
+        return TRUE;
+}
+
+static gboolean
+columns_button_press_event_cb (GtkTreeViewColumn *column,
+                               GdkEventButton *event)
+{
+        if (event->button == 3 && event->type == GDK_BUTTON_PRESS)
+                return columns_show_popup_menu (event, column);
+
+        return FALSE;
+}
+
+static gboolean
+columns_popup_menu_cb (GtkTreeViewColumn *column)
+{
+        return columns_show_popup_menu (NULL, column);
+}
+
+static void
+columns_setup_popup_menu (GtkTreeViewColumn *column)
+{
+        g_return_if_fail (column->button != NULL);
+
+        g_signal_connect_swapped (
+                column->button, "button-press-event",
+                G_CALLBACK (columns_button_press_event_cb), column);
+
+        g_signal_connect_swapped (
+                column->button, "popup-menu",
+                G_CALLBACK (columns_popup_menu_cb), column);
+
+}
+
 /*****************************************************************************
  * Column Factory Callbacks
  *****************************************************************************/
@@ -431,6 +523,46 @@ columns_factory_input_players (GvaGameStoreColumn column_id)
         return column;
 }
 
+static GtkTreeViewColumn *
+columns_factory_input_players_alt (GvaGameStoreColumn column_id)
+{
+#ifdef NPLAYERS_FILE
+        return columns_factory_input_players (column_id);
+#else
+        return NULL;
+#endif
+}
+
+static GtkTreeViewColumn *
+columns_factory_input_players_sim (GvaGameStoreColumn column_id)
+{
+#ifdef NPLAYERS_FILE
+        return columns_factory_input_players (column_id);
+#else
+        return NULL;
+#endif
+}
+
+static GtkTreeViewColumn *
+columns_factory_lastplayed (GvaGameStoreColumn column_id)
+{
+        GtkTreeViewColumn *column;
+        GtkCellRenderer *renderer;
+
+        column = gtk_tree_view_column_new ();
+        gtk_tree_view_column_set_reorderable (column, TRUE);
+        gtk_tree_view_column_set_sort_column_id (column, column_id);
+
+        renderer = gtk_cell_renderer_text_new ();
+        gtk_tree_view_column_pack_start (column, renderer, TRUE);
+
+        gtk_tree_view_column_set_cell_data_func (
+                column, renderer, (GtkTreeCellDataFunc)
+                columns_time_set_properties, NULL, NULL);
+
+        return column;
+}
+
 static GtkTreeViewColumn *
 columns_factory_manufacturer (GvaGameStoreColumn column_id)
 {
@@ -477,7 +609,7 @@ columns_factory_sampleset (GvaGameStoreColumn column_id)
         GdkPixbuf *pixbuf;
 
         /* Remember, we don't own the pixbuf reference. */
-        pixbuf = columns_get_icon_name ("emblem-sound");
+        pixbuf = columns_get_icon_name ("audio-x-generic");
 
         column = gtk_tree_view_column_new ();
         gtk_tree_view_column_set_reorderable (column, TRUE);
@@ -610,7 +742,7 @@ columns_tooltip_driver_status (GtkTreeModel *model,
                 0, GTK_EXPAND | GTK_FILL, 0, 0);
         gtk_widget_show (widget);
 
-        /* The same text is in gnome-video-arcade.glade,
+        /* The same text is in gnome-video-arcade.builder,
          * so it has to be translated with markup anyway. */
         text = _("<b>There are known problems with this game:</b>");
         widget = gtk_label_new (text);
@@ -623,6 +755,7 @@ columns_tooltip_driver_status (GtkTreeModel *model,
 
         /* The labels begin with a UTF-8 encoded bullet character. */
 
+        /* xgettext:no-c-format */
         text = _("• The colors aren't 100% accurate.");
         widget = gtk_label_new (text);
         gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
@@ -641,6 +774,7 @@ columns_tooltip_driver_status (GtkTreeModel *model,
         if (strcmp (driver_color, "preliminary") == 0)
                 gtk_widget_show (widget);
 
+        /* xgettext:no-c-format */
         text = _("• The video emulation isn't 100% accurate.");
         widget = gtk_label_new (text);
         gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
@@ -650,6 +784,7 @@ columns_tooltip_driver_status (GtkTreeModel *model,
         if (strcmp (driver_graphic, "imperfect") == 0)
                 gtk_widget_show (widget);
 
+        /* xgettext:no-c-format */
         text = _("• The sound emulation isn't 100% accurate.");
         widget = gtk_label_new (text);
         gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
@@ -715,11 +850,12 @@ columns_tooltip_favorite (GtkTreeModel *model,
                           GtkTreeIter *iter,
                           GtkTooltip *tooltip)
 {
+        GvaGameStoreColumn column_id;
         const gchar *text;
         gboolean favorite;
 
-        gtk_tree_model_get (
-                model, iter, GVA_GAME_STORE_COLUMN_FAVORITE, &favorite, -1);
+        column_id = GVA_GAME_STORE_COLUMN_FAVORITE;
+        gtk_tree_model_get (model, iter, column_id, &favorite, -1);
 
         if (favorite)
                 text = _("Click here to remove from favorites");
@@ -737,54 +873,66 @@ columns_tooltip_input_players (GtkTreeModel *model,
                                GtkTooltip *tooltip)
 {
         const gchar *text;
+        gint max_alternating;
+        gint max_simultaneous;
         gint max_players;
 
         gtk_tree_model_get (
                 model, iter,
+                GVA_GAME_STORE_COLUMN_INPUT_PLAYERS_ALT, &max_alternating,
+                GVA_GAME_STORE_COLUMN_INPUT_PLAYERS_SIM, &max_simultaneous,
                 GVA_GAME_STORE_COLUMN_INPUT_PLAYERS, &max_players, -1);
 
-        /* Keep this in sync with MAX_PLAYER_ICONS. */
-        switch (max_players)
-        {
-                case 1:
-                        text = _("One player only");
-                        break;
+        /* Fall back to "input_players" if we have to. */
+        if (max_alternating == 0 && max_simultaneous == 0)
+                max_alternating = max_simultaneous = max_players;
 
-                case 2:
-                        text = _("One or two players");
-                        break;
-
-                case 3:
-                        text = _("Up to three players");
-                        break;
-
-                case 4:
-                        text = _("Up to four players");
-                        break;
-
-                case 5:
-                        text = _("Up to five players");
-                        break;
-
-                case 6:
-                        text = _("Up to six players");
-                        break;
-
-                case 7:
-                        text = _("Up to seven players");
-                        break;
-
-                case 8:
-                        text = _("Up to eight players");
-                        break;
-
-                default:
-                        return FALSE;
-        }
+        text = gva_nplayers_describe (max_alternating, max_simultaneous);
 
+        /* This handles NULL for text. */
         gtk_tooltip_set_text (tooltip, text);
 
-        return TRUE;
+        return (text != NULL);
+}
+
+static gboolean
+columns_tooltip_input_players_alt (GtkTreeModel *model,
+                                   GtkTreeIter *iter,
+                                   GtkTooltip *tooltip)
+{
+        GvaGameStoreColumn column_id;
+        const gchar *text;
+        gint max_alternating;
+
+        column_id = GVA_GAME_STORE_COLUMN_INPUT_PLAYERS_ALT;
+        gtk_tree_model_get (model, iter, column_id, &max_alternating, -1);
+
+        text = gva_nplayers_describe (max_alternating, 0);
+
+        /* This handles NULL for text. */
+        gtk_tooltip_set_text (tooltip, text);
+
+        return (text != NULL);
+}
+
+static gboolean
+columns_tooltip_input_players_sim (GtkTreeModel *model,
+                                   GtkTreeIter *iter,
+                                   GtkTooltip *tooltip)
+{
+        GvaGameStoreColumn column_id;
+        const gchar *text;
+        gint max_simultaneous;
+
+        column_id = GVA_GAME_STORE_COLUMN_INPUT_PLAYERS_SIM;
+        gtk_tree_model_get (model, iter, column_id, &max_simultaneous, -1);
+
+        text = gva_nplayers_describe (0, max_simultaneous);
+
+        /* This handles NULL for text. */
+        gtk_tooltip_set_text (tooltip, text);
+
+        return (text != NULL);
 }
 
 static gboolean
@@ -868,6 +1016,12 @@ column_info[GVA_GAME_STORE_NUM_COLUMNS] =
         { "input_players",      N_("Players"),
                                 columns_factory_input_players,
                                 columns_tooltip_input_players },
+        { "input_players_alt",  N_("Players (Alt.)"),
+                                columns_factory_input_players_alt,
+                                columns_tooltip_input_players_alt },
+        { "input_players_sim",  N_("Players (Sim.)"),
+                                columns_factory_input_players_sim,
+                                columns_tooltip_input_players_sim },
         { "input_buttons",      NULL },
         { "input_coins",        NULL },
         { "driver_status",      N_("Status"),
@@ -881,6 +1035,8 @@ column_info[GVA_GAME_STORE_NUM_COLUMNS] =
         { "driver_protection",  NULL },
         { "driver_savestate",   NULL },
         { "driver_palettesize", NULL },
+        { "lastplayed",         N_("Last Played"),
+                                columns_factory_lastplayed },
         { "comment",            N_("Comment"),
                                 columns_factory_comment },
         { "inode",              NULL },
@@ -898,9 +1054,14 @@ static gchar *default_column_order[] =
 #ifdef CATEGORY_FILE
         "category",
 #endif
+        "lastplayed",
         "bios",
         "driver_status",
         "input_players",
+#ifdef NPLAYERS_FILE
+        "input_players_alt",
+        "input_players_sim",
+#endif
         "name",
         "sourcefile",
         "sampleset"
@@ -930,10 +1091,9 @@ gva_columns_new_from_id (GvaGameStoreColumn column_id)
         title = gettext (column_info[column_id].title);
         gtk_tree_view_column_set_title (column, title);
 
-        if (column != NULL)
-                g_object_set_data (
-                        G_OBJECT (column), "name",
-                        (gpointer) column_info[column_id].name);
+        g_object_set_data (
+                G_OBJECT (column), "name",
+                (gpointer) column_info[column_id].name);
 
         return column;
 }
@@ -1114,6 +1274,7 @@ gva_columns_load (GtkTreeView *view)
                 {
                         gtk_tree_view_column_set_visible (column, visible);
                         gtk_tree_view_append_column (view, column);
+                        columns_setup_popup_menu (column);
 
                         g_signal_connect_swapped (
                                 column, "notify::visible",
@@ -1137,6 +1298,7 @@ gva_columns_load (GtkTreeView *view)
                 {
                         gtk_tree_view_column_set_visible (column, TRUE);
                         gtk_tree_view_append_column (view, column);
+                        columns_setup_popup_menu (column);
 
                         g_signal_connect_swapped (
                                 column, "notify::visible",
@@ -1158,6 +1320,7 @@ gva_columns_load (GtkTreeView *view)
                 {
                         gtk_tree_view_column_set_visible (column, FALSE);
                         gtk_tree_view_append_column (view, column);
+                        columns_setup_popup_menu (column);
 
                         g_signal_connect_swapped (
                                 column, "notify::visible",
