@@ -188,6 +188,8 @@ GtkAction *     gva_ui_get_action               (const gchar *action_name);
 GtkWidget *     gva_ui_get_widget               (const gchar *widget_name);
 GtkWidget *     gva_ui_get_managed_widget       (const gchar *widget_path);
 void            gva_ui_add_column_actions       (GtkTreeView *view);
+void            gva_ui_lock                     (void);
+void            gva_ui_unlock                   (void);
 
 G_END_DECLS
 
